use axum::{async_trait, extract::{path::ErrorKind, rejection::PathRejection, FromRequestParts}};
use http::request::Parts;
use i18n::{message, message_ids::MessageId};
use serde::de::DeserializeOwned;

use crate::{context::Context, model::response::ResErr};


pub struct PathVar<T>(pub T);

#[async_trait]
impl<S, T> FromRequestParts<S> for PathVar<T>
where
    T: DeserializeOwned + Send,
    S: Send + Sync,
{
    type Rejection = ResErr;

    async fn from_request_parts(parts: &mut Parts, state: &S) -> Result<Self, Self::Rejection> {
        let context_parts = parts.clone();
        let context: &Context = context_parts.extensions.get().unwrap();
        let lang_tag = context.get_lang_tag();

        match axum::extract::Path::<T>::from_request_parts(parts, state).await {
            Ok(value) => {
                Ok(Self(value.0))
            },
            Err(rejection) => {
                let err_rep = match rejection {
                    PathRejection::FailedToDeserializePathParams(inner) => {
                        let kind = inner.into_kind();
                        let body = match &kind {
                            ErrorKind::WrongNumberOfParameters { got, expected } => {
                                tracing::error!("无效的路径参数: WrongNumberOfParameters, {} - {}", got, expected);
                                ResErr::params(message!(lang_tag, MessageId::InvalidParams))
                            },

                            ErrorKind::ParseErrorAtKey { key, .. } => {
                                tracing::error!("无效的路径参数: ParseErrorAtKey, key: {}", key);
                                ResErr::params(message!(lang_tag, MessageId::InvalidParams))
                            },

                            ErrorKind::ParseErrorAtIndex { index, .. } => {
                                tracing::error!("无效的路径参数: ParseErrorAtIndex, index; {}", index);
                                ResErr::params(message!(lang_tag, MessageId::InvalidParams))
                            },

                            ErrorKind::ParseError { .. } => {
                                tracing::error!("无效的路径参数: ParseError");
                                ResErr::params(message!(lang_tag, MessageId::InvalidParams))
                            },

                            ErrorKind::InvalidUtf8InPathParam { key } => {
                                tracing::error!("无效的路径参数: InvalidUtf8InPathParam, key: {}", key);
                                ResErr::params(message!(lang_tag, MessageId::InvalidParams))
                            },

                            ErrorKind::UnsupportedType { .. } => {
                                // this error is caused by the programmer using an unsupported type
                                // (such as nested maps) so respond with `500` instead
                                tracing::error!("无效的路径参数: UnsupportedType");
                                ResErr::params(message!(lang_tag, MessageId::InvalidParams))
                            }

                            ErrorKind::Message(msg) => {
                                tracing::error!("无效的路径参数: Message, msg: {}", msg);
                                ResErr::params(message!(lang_tag, MessageId::InvalidParams))
                            },

                            _ => {
                                tracing::error!("无效的路径参数");
                                ResErr::params(message!(lang_tag, MessageId::InvalidParams))
                            },
                        };

                        body
                    }
                    PathRejection::MissingPathParams(error) => {
                            tracing::error!("无效的路径参数: MissingPathParams, err: {}", error);
                            ResErr::params(message!(lang_tag, MessageId::InvalidParams))
                    },
                    _ => {
                            tracing::error!("无效的路径参数: Others");
                            ResErr::params(message!(lang_tag, MessageId::InvalidParams))
                        },
                };
                Err(err_rep)
            }
        }
    }
}